@@ -4,6 +4,7 @@ import * as EXPRESSIONS from './regexp.js';
 import {
   addToSet,
   clone,
+  entries,
   freeze,
   arrayForEach,
   arrayPop,
@@ -17,19 +18,23 @@ import {
   regExpTest,
   typeErrorCreate,
   lookupGetter,
+  create,
+  objectHasOwnProperty,
 } from './utils.js';
 
-const getGlobal = () => (typeof window === 'undefined' ? null : window);
+const getGlobal = function () {
+  return typeof window === 'undefined' ? null : window;
+};
 
 /**
  * Creates a no-op policy for internal use only.
  * Don't export this function outside this module!
- * @param {?TrustedTypePolicyFactory} trustedTypes The policy factory.
- * @param {Document} document The document object (to determine policy name suffix)
- * @return {?TrustedTypePolicy} The policy created (or null, if Trusted Types
- * are not supported).
+ * @param {TrustedTypePolicyFactory} trustedTypes The policy factory.
+ * @param {HTMLScriptElement} purifyHostElement The Script element used to load DOMPurify (to determine policy name suffix).
+ * @return {TrustedTypePolicy} The policy created (or null, if Trusted Types
+ * are not supported or creating the policy failed).
  */
-const _createTrustedTypesPolicy = function (trustedTypes, document) {
+const _createTrustedTypesPolicy = function (trustedTypes, purifyHostElement) {
   if (
     typeof trustedTypes !== 'object' ||
     typeof trustedTypes.createPolicy !== 'function'
@@ -42,11 +47,8 @@ const _createTrustedTypesPolicy = function (trustedTypes, document) {
   // Policy creation with duplicate names throws in Trusted Types.
   let suffix = null;
   const ATTR_NAME = 'data-tt-policy-suffix';
-  if (
-    document.currentScript &&
-    document.currentScript.hasAttribute(ATTR_NAME)
-  ) {
-    suffix = document.currentScript.getAttribute(ATTR_NAME);
+  if (purifyHostElement && purifyHostElement.hasAttribute(ATTR_NAME)) {
+    suffix = purifyHostElement.getAttribute(ATTR_NAME);
   }
 
   const policyName = 'dompurify' + (suffix ? '#' + suffix : '');
@@ -94,9 +96,10 @@ function createDOMPurify(window = getGlobal()) {
     return DOMPurify;
   }
 
-  const originalDocument = window.document;
-
   let { document } = window;
+
+  const originalDocument = document;
+  const currentScript = originalDocument.currentScript;
   const {
     DocumentFragment,
     HTMLTemplateElement,
@@ -129,11 +132,8 @@ function createDOMPurify(window = getGlobal()) {
     }
   }
 
-  const trustedTypesPolicy = _createTrustedTypesPolicy(
-    trustedTypes,
-    originalDocument
-  );
-  const emptyHTML = trustedTypesPolicy ? trustedTypesPolicy.createHTML('') : '';
+  let trustedTypesPolicy;
+  let emptyHTML = '';
 
   const {
     implementation,
@@ -143,21 +143,16 @@ function createDOMPurify(window = getGlobal()) {
   } = document;
   const { importNode } = originalDocument;
 
-  let documentMode = {};
-  try {
-    documentMode = clone(document).documentMode ? document.documentMode : {};
-  } catch (_) {}
-
   let hooks = {};
 
   /**
    * Expose whether this browser supports running the full DOMPurify.
    */
   DOMPurify.isSupported =
+    typeof entries === 'function' &&
     typeof getParentNode === 'function' &&
     implementation &&
-    typeof implementation.createHTMLDocument !== 'undefined' &&
-    documentMode !== 9;
+    implementation.createHTMLDocument !== undefined;
 
   const {
     MUSTACHE_EXPR,
@@ -201,8 +196,8 @@ function createDOMPurify(window = getGlobal()) {
    * @property {RegExp|Function|null} attributeNameCheck one of [null, regexPattern, predicate]. Default: `null` (disallow any attributes not on the allow list)
    * @property {boolean} allowCustomizedBuiltInElements allow custom elements derived from built-ins if they pass CUSTOM_ELEMENT_HANDLING.tagNameCheck. Default: `false`.
    */
-  const CUSTOM_ELEMENT_HANDLING = Object.seal(
-    Object.create(null, {
+  let CUSTOM_ELEMENT_HANDLING = Object.seal(
+    create(null, {
       tagNameCheck: {
         writable: true,
         configurable: false,
@@ -239,6 +234,10 @@ function createDOMPurify(window = getGlobal()) {
   /* Decide if unknown protocols are okay */
   let ALLOW_UNKNOWN_PROTOCOLS = false;
 
+  /* Decide if self-closing tags in attributes are allowed.
+   * Usually removed due to a mXSS issue in jQuery 3.0 */
+  let ALLOW_SELF_CLOSE_IN_ATTR = true;
+
   /* Output should be safe for common template engines.
    * This means, DOMPurify removes data attributes, mustaches and ERB
    */
@@ -375,10 +374,10 @@ function createDOMPurify(window = getGlobal()) {
   );
 
   /* Parsing of strict XHTML documents */
-  let PARSER_MEDIA_TYPE;
+  let PARSER_MEDIA_TYPE = null;
   const SUPPORTED_PARSER_MEDIA_TYPES = ['application/xhtml+xml', 'text/html'];
   const DEFAULT_PARSER_MEDIA_TYPE = 'text/html';
-  let transformCaseFunc;
+  let transformCaseFunc = null;
 
   /* Keep a reference to config to pass to hooks */
   let CONFIG = null;
@@ -398,7 +397,7 @@ function createDOMPurify(window = getGlobal()) {
    * @param  {Object} cfg optional config literal
    */
   // eslint-disable-next-line complexity
-  const _parseConfig = function (cfg) {
+  const _parseConfig = function (cfg = {}) {
     if (CONFIG && CONFIG === cfg) {
       return;
     }
@@ -414,8 +413,8 @@ function createDOMPurify(window = getGlobal()) {
     PARSER_MEDIA_TYPE =
       // eslint-disable-next-line unicorn/prefer-includes
       SUPPORTED_PARSER_MEDIA_TYPES.indexOf(cfg.PARSER_MEDIA_TYPE) === -1
-        ? (PARSER_MEDIA_TYPE = DEFAULT_PARSER_MEDIA_TYPE)
-        : (PARSER_MEDIA_TYPE = cfg.PARSER_MEDIA_TYPE);
+        ? DEFAULT_PARSER_MEDIA_TYPE
+        : cfg.PARSER_MEDIA_TYPE;
 
     // HTML tags and attributes are not case-sensitive, converting to lowercase. Keeping XHTML as is.
     transformCaseFunc =
@@ -424,50 +423,45 @@ function createDOMPurify(window = getGlobal()) {
         : stringToLowerCase;
 
     /* Set configuration parameters */
-    ALLOWED_TAGS =
-      'ALLOWED_TAGS' in cfg
-        ? addToSet({}, cfg.ALLOWED_TAGS, transformCaseFunc)
-        : DEFAULT_ALLOWED_TAGS;
-    ALLOWED_ATTR =
-      'ALLOWED_ATTR' in cfg
-        ? addToSet({}, cfg.ALLOWED_ATTR, transformCaseFunc)
-        : DEFAULT_ALLOWED_ATTR;
-    ALLOWED_NAMESPACES =
-      'ALLOWED_NAMESPACES' in cfg
-        ? addToSet({}, cfg.ALLOWED_NAMESPACES, stringToString)
-        : DEFAULT_ALLOWED_NAMESPACES;
-    URI_SAFE_ATTRIBUTES =
-      'ADD_URI_SAFE_ATTR' in cfg
-        ? addToSet(
-            clone(DEFAULT_URI_SAFE_ATTRIBUTES), // eslint-disable-line indent
-            cfg.ADD_URI_SAFE_ATTR, // eslint-disable-line indent
-            transformCaseFunc // eslint-disable-line indent
-          ) // eslint-disable-line indent
-        : DEFAULT_URI_SAFE_ATTRIBUTES;
-    DATA_URI_TAGS =
-      'ADD_DATA_URI_TAGS' in cfg
-        ? addToSet(
-            clone(DEFAULT_DATA_URI_TAGS), // eslint-disable-line indent
-            cfg.ADD_DATA_URI_TAGS, // eslint-disable-line indent
-            transformCaseFunc // eslint-disable-line indent
-          ) // eslint-disable-line indent
-        : DEFAULT_DATA_URI_TAGS;
-    FORBID_CONTENTS =
-      'FORBID_CONTENTS' in cfg
-        ? addToSet({}, cfg.FORBID_CONTENTS, transformCaseFunc)
-        : DEFAULT_FORBID_CONTENTS;
-    FORBID_TAGS =
-      'FORBID_TAGS' in cfg
-        ? addToSet({}, cfg.FORBID_TAGS, transformCaseFunc)
-        : {};
-    FORBID_ATTR =
-      'FORBID_ATTR' in cfg
-        ? addToSet({}, cfg.FORBID_ATTR, transformCaseFunc)
-        : {};
-    USE_PROFILES = 'USE_PROFILES' in cfg ? cfg.USE_PROFILES : false;
+    ALLOWED_TAGS = objectHasOwnProperty(cfg, 'ALLOWED_TAGS')
+      ? addToSet({}, cfg.ALLOWED_TAGS, transformCaseFunc)
+      : DEFAULT_ALLOWED_TAGS;
+    ALLOWED_ATTR = objectHasOwnProperty(cfg, 'ALLOWED_ATTR')
+      ? addToSet({}, cfg.ALLOWED_ATTR, transformCaseFunc)
+      : DEFAULT_ALLOWED_ATTR;
+    ALLOWED_NAMESPACES = objectHasOwnProperty(cfg, 'ALLOWED_NAMESPACES')
+      ? addToSet({}, cfg.ALLOWED_NAMESPACES, stringToString)
+      : DEFAULT_ALLOWED_NAMESPACES;
+    URI_SAFE_ATTRIBUTES = objectHasOwnProperty(cfg, 'ADD_URI_SAFE_ATTR')
+      ? addToSet(
+          clone(DEFAULT_URI_SAFE_ATTRIBUTES), // eslint-disable-line indent
+          cfg.ADD_URI_SAFE_ATTR, // eslint-disable-line indent
+          transformCaseFunc // eslint-disable-line indent
+        ) // eslint-disable-line indent
+      : DEFAULT_URI_SAFE_ATTRIBUTES;
+    DATA_URI_TAGS = objectHasOwnProperty(cfg, 'ADD_DATA_URI_TAGS')
+      ? addToSet(
+          clone(DEFAULT_DATA_URI_TAGS), // eslint-disable-line indent
+          cfg.ADD_DATA_URI_TAGS, // eslint-disable-line indent
+          transformCaseFunc // eslint-disable-line indent
+        ) // eslint-disable-line indent
+      : DEFAULT_DATA_URI_TAGS;
+    FORBID_CONTENTS = objectHasOwnProperty(cfg, 'FORBID_CONTENTS')
+      ? addToSet({}, cfg.FORBID_CONTENTS, transformCaseFunc)
+      : DEFAULT_FORBID_CONTENTS;
+    FORBID_TAGS = objectHasOwnProperty(cfg, 'FORBID_TAGS')
+      ? addToSet({}, cfg.FORBID_TAGS, transformCaseFunc)
+      : {};
+    FORBID_ATTR = objectHasOwnProperty(cfg, 'FORBID_ATTR')
+      ? addToSet({}, cfg.FORBID_ATTR, transformCaseFunc)
+      : {};
+    USE_PROFILES = objectHasOwnProperty(cfg, 'USE_PROFILES')
+      ? cfg.USE_PROFILES
+      : false;
     ALLOW_ARIA_ATTR = cfg.ALLOW_ARIA_ATTR !== false; // Default true
     ALLOW_DATA_ATTR = cfg.ALLOW_DATA_ATTR !== false; // Default true
     ALLOW_UNKNOWN_PROTOCOLS = cfg.ALLOW_UNKNOWN_PROTOCOLS || false; // Default false
+    ALLOW_SELF_CLOSE_IN_ATTR = cfg.ALLOW_SELF_CLOSE_IN_ATTR !== false; // Default true
     SAFE_FOR_TEMPLATES = cfg.SAFE_FOR_TEMPLATES || false; // Default false
     WHOLE_DOCUMENT = cfg.WHOLE_DOCUMENT || false; // Default false
     RETURN_DOM = cfg.RETURN_DOM || false; // Default false
@@ -478,8 +472,9 @@ function createDOMPurify(window = getGlobal()) {
     SANITIZE_NAMED_PROPS = cfg.SANITIZE_NAMED_PROPS || false; // Default false
     KEEP_CONTENT = cfg.KEEP_CONTENT !== false; // Default true
     IN_PLACE = cfg.IN_PLACE || false; // Default false
-    IS_ALLOWED_URI = cfg.ALLOWED_URI_REGEXP || IS_ALLOWED_URI;
+    IS_ALLOWED_URI = cfg.ALLOWED_URI_REGEXP || EXPRESSIONS.IS_ALLOWED_URI;
     NAMESPACE = cfg.NAMESPACE || HTML_NAMESPACE;
+    CUSTOM_ELEMENT_HANDLING = cfg.CUSTOM_ELEMENT_HANDLING || {};
     if (
       cfg.CUSTOM_ELEMENT_HANDLING &&
       isRegexOrFunction(cfg.CUSTOM_ELEMENT_HANDLING.tagNameCheck)
@@ -515,7 +510,7 @@ function createDOMPurify(window = getGlobal()) {
 
     /* Parse profile info */
     if (USE_PROFILES) {
-      ALLOWED_TAGS = addToSet({}, [...TAGS.text]);
+      ALLOWED_TAGS = addToSet({}, TAGS.text);
       ALLOWED_ATTR = [];
       if (USE_PROFILES.html === true) {
         addToSet(ALLOWED_TAGS, TAGS.html);
@@ -586,6 +581,39 @@ function createDOMPurify(window = getGlobal()) {
       delete FORBID_TAGS.tbody;
     }
 
+    if (cfg.TRUSTED_TYPES_POLICY) {
+      if (typeof cfg.TRUSTED_TYPES_POLICY.createHTML !== 'function') {
+        throw typeErrorCreate(
+          'TRUSTED_TYPES_POLICY configuration option must provide a "createHTML" hook.'
+        );
+      }
+
+      if (typeof cfg.TRUSTED_TYPES_POLICY.createScriptURL !== 'function') {
+        throw typeErrorCreate(
+          'TRUSTED_TYPES_POLICY configuration option must provide a "createScriptURL" hook.'
+        );
+      }
+
+      // Overwrite existing TrustedTypes policy.
+      trustedTypesPolicy = cfg.TRUSTED_TYPES_POLICY;
+
+      // Sign local variables required by `sanitize`.
+      emptyHTML = trustedTypesPolicy.createHTML('');
+    } else {
+      // Uninitialized policy, attempt to initialize the internal dompurify policy.
+      if (trustedTypesPolicy === undefined) {
+        trustedTypesPolicy = _createTrustedTypesPolicy(
+          trustedTypes,
+          currentScript
+        );
+      }
+
+      // If creating the internal policy succeeded sign internal variables.
+      if (trustedTypesPolicy !== null && typeof emptyHTML === 'string') {
+        emptyHTML = trustedTypesPolicy.createHTML('');
+      }
+    }
+
     // Prevent further manipulation of configuration.
     // Not available in IE8, Safari 5, etc.
     if (freeze) {
@@ -625,16 +653,17 @@ function createDOMPurify(window = getGlobal()) {
   /* Keep track of all possible SVG and MathML tags
    * so that we can perform the namespace checks
    * correctly. */
-  const ALL_SVG_TAGS = addToSet({}, TAGS.svg);
-  addToSet(ALL_SVG_TAGS, TAGS.svgFilters);
-  addToSet(ALL_SVG_TAGS, TAGS.svgDisallowed);
-
-  const ALL_MATHML_TAGS = addToSet({}, TAGS.mathMl);
-  addToSet(ALL_MATHML_TAGS, TAGS.mathMlDisallowed);
+  const ALL_SVG_TAGS = addToSet({}, [
+    ...TAGS.svg,
+    ...TAGS.svgFilters,
+    ...TAGS.svgDisallowed,
+  ]);
+  const ALL_MATHML_TAGS = addToSet({}, [
+    ...TAGS.mathMl,
+    ...TAGS.mathMlDisallowed,
+  ]);
 
   /**
-   *
-   *
    * @param  {Element} element a DOM element whose namespace is being checked
    * @returns {boolean} Return false if the element has a
    *  namespace that a spec-compliant parser would never
@@ -750,15 +779,12 @@ function createDOMPurify(window = getGlobal()) {
    */
   const _forceRemove = function (node) {
     arrayPush(DOMPurify.removed, { element: node });
+
     try {
       // eslint-disable-next-line unicorn/prefer-dom-node-remove
       node.parentNode.removeChild(node);
     } catch (_) {
-      try {
-        node.outerHTML = emptyHTML;
-      } catch (_) {
-        node.remove();
-      }
+      node.remove();
     }
   };
 
@@ -805,8 +831,8 @@ function createDOMPurify(window = getGlobal()) {
    */
   const _initDocument = function (dirty) {
     /* Create a HTML document */
-    let doc;
-    let leadingWhitespace;
+    let doc = null;
+    let leadingWhitespace = null;
 
     if (FORCE_BODY) {
       dirty = '<remove></remove>' + dirty;
@@ -844,7 +870,9 @@ function createDOMPurify(window = getGlobal()) {
     if (!doc || !doc.documentElement) {
       doc = implementation.createDocument(NAMESPACE, 'template', null);
       try {
-        doc.documentElement.innerHTML = IS_EMPTY_INPUT ? '' : dirtyPayload;
+        doc.documentElement.innerHTML = IS_EMPTY_INPUT
+          ? emptyHTML
+          : dirtyPayload;
       } catch (_) {
         // Syntax error if dirtyPayload is invalid xml
       }
@@ -871,19 +899,18 @@ function createDOMPurify(window = getGlobal()) {
   };
 
   /**
-   * _createIterator
+   * Creates a NodeIterator object that you can use to traverse filtered lists of nodes or elements in a document.
    *
-   * @param  {Document} root document/fragment to create iterator for
-   * @return {Iterator} iterator instance
+   * @param  {Node} root The root element or node to start traversing on.
+   * @return {NodeIterator} The created NodeIterator
    */
-  const _createIterator = function (root) {
+  const _createNodeIterator = function (root) {
     return createNodeIterator.call(
       root.ownerDocument || root,
       root,
       // eslint-disable-next-line no-bitwise
       NodeFilter.SHOW_ELEMENT | NodeFilter.SHOW_COMMENT | NodeFilter.SHOW_TEXT,
-      null,
-      false
+      null
     );
   };
 
@@ -909,18 +936,13 @@ function createDOMPurify(window = getGlobal()) {
   };
 
   /**
-   * _isNode
+   * Checks whether the given object is a DOM node.
    *
-   * @param  {Node} obj object to check whether it's a DOM node
+   * @param  {Node} object object to check whether it's a DOM node
    * @return {Boolean} true is object is a DOM node
    */
   const _isNode = function (object) {
-    return typeof Node === 'object'
-      ? object instanceof Node
-      : object &&
-          typeof object === 'object' &&
-          typeof object.nodeType === 'number' &&
-          typeof object.nodeName === 'string';
+    return typeof Node === 'function' && object instanceof Node;
   };
 
   /**
@@ -952,7 +974,7 @@ function createDOMPurify(window = getGlobal()) {
    * @return  {Boolean} true if node was killed, false if left alive
    */
   const _sanitizeElements = function (currentNode) {
-    let content;
+    let content = null;
 
     /* Execute a hook if present */
     _executeHook('beforeSanitizeElements', currentNode, null);
@@ -963,12 +985,6 @@ function createDOMPurify(window = getGlobal()) {
       return true;
     }
 
-    /* Check if tagname contains Unicode */
-    if (regExpTest(/[\u0080-\uFFFF]/, currentNode.nodeName)) {
-      _forceRemove(currentNode);
-      return true;
-    }
-
     /* Now let's check the element's type and name */
     const tagName = transformCaseFunc(currentNode.nodeName);
 
@@ -982,8 +998,6 @@ function createDOMPurify(window = getGlobal()) {
     if (
       currentNode.hasChildNodes() &&
       !_isNode(currentNode.firstElementChild) &&
-      (!_isNode(currentNode.content) ||
-        !_isNode(currentNode.content.firstElementChild)) &&
       regExpTest(/<[/\w]/g, currentNode.innerHTML) &&
       regExpTest(/<[/\w]/g, currentNode.textContent)
     ) {
@@ -991,29 +1005,23 @@ function createDOMPurify(window = getGlobal()) {
       return true;
     }
 
-    /* Mitigate a problem with templates inside select */
-    if (
-      tagName === 'select' &&
-      regExpTest(/<template/i, currentNode.innerHTML)
-    ) {
-      _forceRemove(currentNode);
-      return true;
-    }
-
     /* Remove element if anything forbids its presence */
     if (!ALLOWED_TAGS[tagName] || FORBID_TAGS[tagName]) {
       /* Check if we have a custom element to handle */
-      if (!FORBID_TAGS[tagName] && _basicCustomElementTest(tagName)) {
+      if (!FORBID_TAGS[tagName] && _isBasicCustomElement(tagName)) {
         if (
           CUSTOM_ELEMENT_HANDLING.tagNameCheck instanceof RegExp &&
           regExpTest(CUSTOM_ELEMENT_HANDLING.tagNameCheck, tagName)
-        )
+        ) {
           return false;
+        }
+
         if (
           CUSTOM_ELEMENT_HANDLING.tagNameCheck instanceof Function &&
           CUSTOM_ELEMENT_HANDLING.tagNameCheck(tagName)
-        )
+        ) {
           return false;
+        }
       }
 
       /* Keep content except for bad-listed elements */
@@ -1043,9 +1051,12 @@ function createDOMPurify(window = getGlobal()) {
       return true;
     }
 
+    /* Make sure that older browsers don't get fallback-tag mXSS */
     if (
-      (tagName === 'noscript' || tagName === 'noembed') &&
-      regExpTest(/<\/no(script|embed)/i, currentNode.innerHTML)
+      (tagName === 'noscript' ||
+        tagName === 'noembed' ||
+        tagName === 'noframes') &&
+      regExpTest(/<\/no(script|embed|frames)/i, currentNode.innerHTML)
     ) {
       _forceRemove(currentNode);
       return true;
@@ -1055,9 +1066,11 @@ function createDOMPurify(window = getGlobal()) {
     if (SAFE_FOR_TEMPLATES && currentNode.nodeType === 3) {
       /* Get the element's text content */
       content = currentNode.textContent;
-      content = stringReplace(content, MUSTACHE_EXPR, ' ');
-      content = stringReplace(content, ERB_EXPR, ' ');
-      content = stringReplace(content, TMPLIT_EXPR, ' ');
+
+      arrayForEach([MUSTACHE_EXPR, ERB_EXPR, TMPLIT_EXPR], (expr) => {
+        content = stringReplace(content, expr, ' ');
+      });
+
       if (currentNode.textContent !== content) {
         arrayPush(DOMPurify.removed, { element: currentNode.cloneNode() });
         currentNode.textContent = content;
@@ -1107,7 +1120,7 @@ function createDOMPurify(window = getGlobal()) {
         // First condition does a very basic check if a) it's basically a valid custom element tagname AND
         // b) if the tagName passes whatever the user has configured for CUSTOM_ELEMENT_HANDLING.tagNameCheck
         // and c) if the attribute name passes whatever the user has configured for CUSTOM_ELEMENT_HANDLING.attributeNameCheck
-        (_basicCustomElementTest(lcTag) &&
+        (_isBasicCustomElement(lcTag) &&
           ((CUSTOM_ELEMENT_HANDLING.tagNameCheck instanceof RegExp &&
             regExpTest(CUSTOM_ELEMENT_HANDLING.tagNameCheck, lcTag)) ||
             (CUSTOM_ELEMENT_HANDLING.tagNameCheck instanceof Function &&
@@ -1157,25 +1170,26 @@ function createDOMPurify(window = getGlobal()) {
     ) {
       // This attribute is safe
       /* Check for binary attributes */
-      // eslint-disable-next-line no-negated-condition
-    } else if (!value) {
+    } else if (value) {
+      return false;
+    } else {
       // Binary attributes are safe at this point
       /* Anything else, presume unsafe, do not add it back */
-    } else {
-      return false;
     }
 
     return true;
   };
 
   /**
-   * _basicCustomElementCheck
+   * _isBasicCustomElement
    * checks if at least one dash is included in tagName, and it's not the first char
    * for more sophisticated checking see https://github.com/sindresorhus/validate-element-name
+   *
    * @param {string} tagName name of the tag of the node to sanitize
+   * @returns {boolean} Returns true if the tag name meets the basic criteria for a custom element, otherwise false.
    */
-  const _basicCustomElementTest = function (tagName) {
-    return tagName.indexOf('-') > 0;
+  const _isBasicCustomElement = function (tagName) {
+    return tagName !== 'annotation-xml' && tagName.indexOf('-') > 0;
   };
 
   /**
@@ -1189,10 +1203,6 @@ function createDOMPurify(window = getGlobal()) {
    * @param  {Node} currentNode to sanitize
    */
   const _sanitizeAttributes = function (currentNode) {
-    let attr;
-    let value;
-    let lcName;
-    let l;
     /* Execute a hook if present */
     _executeHook('beforeSanitizeAttributes', currentNode, null);
 
@@ -1209,14 +1219,15 @@ function createDOMPurify(window = getGlobal()) {
       keepAttr: true,
       allowedAttributes: ALLOWED_ATTR,
     };
-    l = attributes.length;
+    let l = attributes.length;
 
     /* Go backwards over all attributes; safely remove bad ones */
     while (l--) {
-      attr = attributes[l];
-      const { name, namespaceURI } = attr;
-      value = name === 'value' ? attr.value : stringTrim(attr.value);
-      lcName = transformCaseFunc(name);
+      const attr = attributes[l];
+      const { name, namespaceURI, value: attrValue } = attr;
+      const lcName = transformCaseFunc(name);
+
+      let value = name === 'value' ? attrValue : stringTrim(attrValue);
 
       /* Execute a hook if present */
       hookEvent.attrName = lcName;
@@ -1239,16 +1250,16 @@ function createDOMPurify(window = getGlobal()) {
       }
 
       /* Work around a security issue in jQuery 3.0 */
-      if (regExpTest(/\/>/i, value)) {
+      if (!ALLOW_SELF_CLOSE_IN_ATTR && regExpTest(/\/>/i, value)) {
         _removeAttribute(name, currentNode);
         continue;
       }
 
       /* Sanitize attribute content to be template-safe */
       if (SAFE_FOR_TEMPLATES) {
-        value = stringReplace(value, MUSTACHE_EXPR, ' ');
-        value = stringReplace(value, ERB_EXPR, ' ');
-        value = stringReplace(value, TMPLIT_EXPR, ' ');
+        arrayForEach([MUSTACHE_EXPR, ERB_EXPR, TMPLIT_EXPR], (expr) => {
+          value = stringReplace(value, expr, ' ');
+        });
       }
 
       /* Is `value` valid for this attribute? */
@@ -1278,14 +1289,19 @@ function createDOMPurify(window = getGlobal()) {
           /* Namespaces are not yet supported, see https://bugs.chromium.org/p/chromium/issues/detail?id=1305293 */
         } else {
           switch (trustedTypes.getAttributeType(lcTag, lcName)) {
-            case 'TrustedHTML':
+            case 'TrustedHTML': {
               value = trustedTypesPolicy.createHTML(value);
               break;
-            case 'TrustedScriptURL':
+            }
+
+            case 'TrustedScriptURL': {
               value = trustedTypesPolicy.createScriptURL(value);
               break;
-            default:
+            }
+
+            default: {
               break;
+            }
           }
         }
       }
@@ -1313,8 +1329,8 @@ function createDOMPurify(window = getGlobal()) {
    * @param  {DocumentFragment} fragment to iterate over recursively
    */
   const _sanitizeShadowDOM = function (fragment) {
-    let shadowNode;
-    const shadowIterator = _createIterator(fragment);
+    let shadowNode = null;
+    const shadowIterator = _createNodeIterator(fragment);
 
     /* Execute a hook if present */
     _executeHook('beforeSanitizeShadowDOM', fragment, null);
@@ -1346,15 +1362,14 @@ function createDOMPurify(window = getGlobal()) {
    * Public method providing core sanitation functionality
    *
    * @param {String|Node} dirty string or DOM node
-   * @param {Object} configuration object
+   * @param {Object} cfg object
    */
   // eslint-disable-next-line complexity
   DOMPurify.sanitize = function (dirty, cfg = {}) {
-    let body;
-    let importedNode;
-    let currentNode;
-    let oldNode;
-    let returnNode;
+    let body = null;
+    let importedNode = null;
+    let currentNode = null;
+    let returnNode = null;
     /* Make sure we have a string to sanitize.
       DO NOT return early, as this will return the wrong type if
       the user has requested a DOM object rather than a string */
@@ -1365,32 +1380,18 @@ function createDOMPurify(window = getGlobal()) {
 
     /* Stringify, in case dirty is an object */
     if (typeof dirty !== 'string' && !_isNode(dirty)) {
-      // eslint-disable-next-line no-negated-condition
-      if (typeof dirty.toString !== 'function') {
-        throw typeErrorCreate('toString is not a function');
-      } else {
+      if (typeof dirty.toString === 'function') {
         dirty = dirty.toString();
         if (typeof dirty !== 'string') {
           throw typeErrorCreate('dirty is not a string, aborting');
         }
+      } else {
+        throw typeErrorCreate('toString is not a function');
       }
     }
 
-    /* Check we can run. Otherwise fall back or ignore */
+    /* Return dirty HTML if DOMPurify cannot run */
     if (!DOMPurify.isSupported) {
-      if (
-        typeof window.toStaticHTML === 'object' ||
-        typeof window.toStaticHTML === 'function'
-      ) {
-        if (typeof dirty === 'string') {
-          return window.toStaticHTML(dirty);
-        }
-
-        if (_isNode(dirty)) {
-          return window.toStaticHTML(dirty.outerHTML);
-        }
-      }
-
       return dirty;
     }
 
@@ -1460,15 +1461,10 @@ function createDOMPurify(window = getGlobal()) {
     }
 
     /* Get node iterator */
-    const nodeIterator = _createIterator(IN_PLACE ? dirty : body);
+    const nodeIterator = _createNodeIterator(IN_PLACE ? dirty : body);
 
     /* Now start iterating over the created document */
     while ((currentNode = nodeIterator.nextNode())) {
-      /* Fix IE's strange behavior with manipulated textNodes #89 */
-      if (currentNode.nodeType === 3 && currentNode === oldNode) {
-        continue;
-      }
-
       /* Sanitize tags and elements */
       if (_sanitizeElements(currentNode)) {
         continue;
@@ -1481,12 +1477,8 @@ function createDOMPurify(window = getGlobal()) {
 
       /* Check attributes, sanitize if necessary */
       _sanitizeAttributes(currentNode);
-
-      oldNode = currentNode;
     }
 
-    oldNode = null;
-
     /* If we sanitized `dirty` in-place, return it. */
     if (IN_PLACE) {
       return dirty;
@@ -1505,7 +1497,7 @@ function createDOMPurify(window = getGlobal()) {
         returnNode = body;
       }
 
-      if (ALLOWED_ATTR.shadowroot) {
+      if (ALLOWED_ATTR.shadowroot || ALLOWED_ATTR.shadowrootmode) {
         /*
           AdoptNode() is not used because internal state is not reset
           (e.g. the past names map of a HTMLFormElement), this is safe
@@ -1536,9 +1528,9 @@ function createDOMPurify(window = getGlobal()) {
 
     /* Sanitize final string template-safe */
     if (SAFE_FOR_TEMPLATES) {
-      serializedHTML = stringReplace(serializedHTML, MUSTACHE_EXPR, ' ');
-      serializedHTML = stringReplace(serializedHTML, ERB_EXPR, ' ');
-      serializedHTML = stringReplace(serializedHTML, TMPLIT_EXPR, ' ');
+      arrayForEach([MUSTACHE_EXPR, ERB_EXPR, TMPLIT_EXPR], (expr) => {
+        serializedHTML = stringReplace(serializedHTML, expr, ' ');
+      });
     }
 
     return trustedTypesPolicy && RETURN_TRUSTED_TYPE
@@ -1552,7 +1544,7 @@ function createDOMPurify(window = getGlobal()) {
    *
    * @param {Object} cfg configuration object
    */
-  DOMPurify.setConfig = function (cfg) {
+  DOMPurify.setConfig = function (cfg = {}) {
     _parseConfig(cfg);
     SET_CONFIG = true;
   };
@@ -1572,9 +1564,9 @@ function createDOMPurify(window = getGlobal()) {
    * Uses last set config, if any. Otherwise, uses config defaults.
    * isValidAttribute
    *
-   * @param  {string} tag Tag name of containing element.
-   * @param  {string} attr Attribute name.
-   * @param  {string} value Attribute value.
+   * @param  {String} tag Tag name of containing element.
+   * @param  {String} attr Attribute name.
+   * @param  {String} value Attribute value.
    * @return {Boolean} Returns true if `value` is valid. Otherwise, returns false.
    */
   DOMPurify.isValidAttribute = function (tag, attr, value) {
@@ -1633,7 +1625,6 @@ function createDOMPurify(window = getGlobal()) {
   /**
    * RemoveAllHooks
    * Public method to remove all DOMPurify hooks
-   *
    */
   DOMPurify.removeAllHooks = function () {
     hooks = {};
