@@ -2,7 +2,7 @@
 /* global QUnit */
 'use strict';
 
-global.QUnit = require('qunitjs');
+global.QUnit = require('qunit');
 
 const qunitTap = require('qunit-tap');
 const argument = process.argv[2];
@@ -20,6 +20,5 @@ qunitTap(QUnit, (line) => {
   console.log(line);
 });
 
-require('./jsdom-node');
-
-QUnit.load();
+const startQUnit = require('./jsdom-node');
+startQUnit().then(() => QUnit.load());
