@@ -133,6 +133,21 @@
         );
       }
     );
+    QUnit.test('Config-Flag tests: ALLOW_SELF_CLOSE_IN_ATTR', function (assert) {
+      // ALLOW_SELF_CLOSE_IN_ATTR
+      assert.equal(
+        DOMPurify.sanitize('<a href="#" class="foo <br/>">abc</a>', {
+          ALLOW_SELF_CLOSE_IN_ATTR: false,
+        }),
+        '<a href="#">abc</a>'
+      );
+      assert.equal(
+        DOMPurify.sanitize('<a href="#" class="foo <br/>">abc</a>', {
+          ALLOW_SELF_CLOSE_IN_ATTR: true,
+        }),
+        '<a class="foo <br/>" href="#">abc</a>'
+      );
+    });
     QUnit.test('Config-Flag tests: ALLOW_DATA_ATTR', function (assert) {
       // ALLOW_DATA_ATTR
       assert.equal(
@@ -789,6 +804,21 @@
         );
       }
     );
+    QUnit.test(
+      'CUSTOM_ELEMENT_HANDLING config values of null do not throw a TypeError.',
+      function (assert) {
+        DOMPurify.sanitize('', {
+          CUSTOM_ELEMENT_HANDLING: {
+            tagNameCheck: null,
+            attributeNameCheck: null,
+            allowCustomizedBuiltInElements: null,
+          },
+        });
+    
+        // Don't see a great way to assert NOT throws...
+        assert.ok(true);
+      }
+    );
     QUnit.test('Test dirty being an array', function (assert) {
       assert.equal(
         DOMPurify.sanitize(['<a>123<b>456</b></a>']),
@@ -870,23 +900,6 @@
         assert.equal(resultBody, '123<img>');
       }
     );
-    // Test to check against a hang in MSIE (#89)
-    QUnit.test(
-      'sanitize() should not hang on MSIE when hook changes textContent',
-      function (assert) {
-        DOMPurify.addHook('afterSanitizeElements', function (node) {
-          if (node.nodeType && node.nodeType === document.TEXT_NODE) {
-            node.textContent = 'foo';
-          }
-          return node;
-        });
-        var dirty =
-          '<div><p>This is a beatufiul text</p><p>This is too</p></div>';
-        var modified = '<div><p>foo</p><p>foo</p></div>';
-        assert.equal(DOMPurify.sanitize(dirty), modified);
-        DOMPurify.removeHooks('afterSanitizeElements');
-      }
-    );
     // Tests to ensure that a configuration can be set and cleared
     QUnit.test(
       'ensure that a persistent configuration can be set and cleared',
@@ -1175,17 +1188,6 @@
         ]);
       }
     );
-    // Test to make sure that empty HTML doesn't return null on MSIE11 (#198)
-    QUnit.test(
-      "Empty HTML shouldn't return null on MSIE11 in RETURN_DOM_FRAGMENT mode",
-      function (assert) {
-        var clean = DOMPurify.sanitize('', {
-          RETURN_DOM: true,
-          RETURN_DOM_FRAGMENT: true,
-        });
-        assert.equal(typeof clean, 'object');
-      }
-    );
     // Tests to make sure that FORCE_BODY pushes elements to document.body (#199)
     QUnit.test(
       'FORCE_BODY needs to push some elements to document.body',
@@ -1429,6 +1431,21 @@
         assert.equal(str, test.expected);
       });
     });
+    QUnit.test('Ensure ALLOWED_URI_REGEXP is not cached', function(assert) {
+      const 
+        dirty = '<img src="https://different.com">',
+        expected = '<img src="https://different.com">';
+
+      assert.equal(DOMPurify.sanitize(dirty), expected);
+
+      // sanitize with a custom URI regexp
+      assert.equal(DOMPurify.sanitize('<img src="https://test.com">', {
+        ALLOWED_URI_REGEXP: /test\.com/i
+      }), '<img src="https://test.com">');
+
+      // ensure that the previous regexp does not affect future santize calls
+      assert.equal(DOMPurify.sanitize(dirty), expected);
+    });
     QUnit.test(
       'Avoid freeze when using tables and ALLOW_TAGS',
       function (assert) {
@@ -1686,21 +1703,6 @@
         ]);
       }
     );
-    QUnit.test(
-      'Test protection from prototype pollution attacks',
-      function (assert) {
-        const obj = JSON.parse(
-          '{"ALLOWED_ATTR":["onerror","src"], "documentMode":9}'
-        );
-        try {
-          for (let [k, v] of Object.entries(obj)) {
-            Object.prototype[k] = v;
-          }
-        } catch (e) {}
-        var clean = DOMPurify.sanitize('<img src=x onerror=alert(1)>');
-        assert.equal(clean, '<img src="x">');
-      }
-    );
     QUnit.test('Test if namespaces are properly enforced', function (assert) {
       var tests = [
         {
@@ -1774,7 +1776,7 @@
         {
           test: '<polyline points="0 0"></polyline>',
           config: { NAMESPACE: 'http://www.w3.org/1999/xhtml' },
-          expected: ['', '<polyline points="0 0"></polyline>'], // IE10
+          expected: [''],
         },
         {
           test: '<mi></mi>',
@@ -1790,7 +1792,7 @@
         {
           test: '<polyline points="0 0"></polyline>',
           config: { NAMESPACE: 'http://www.w3.org/1998/Math/MathML' },
-          expected: ['', '<polyline points="0 0"></polyline>'], // IE10
+          expected: [''],
         },
         {
           test: '<mi></mi>',
@@ -2087,6 +2089,20 @@
       // set the same hook
       DOMPurify.addHook(entryPoint, hookFunction);
       assert.equal(DOMPurify.sanitize(dirty), expected);
+
+      // cleanup hook
+      DOMPurify.removeHook(entryPoint);
+    });
+    
+    QUnit.test('Test proper removal of annotation-xml w. custom elements', function (assert) {
+      const dirty  = '<svg><annotation-xml><foreignobject><style><!--</style><p id="--><img src=\'x\' onerror=\'alert(1)\'>">';
+      const config = { 
+        CUSTOM_ELEMENT_HANDLING: { tagNameCheck: /.*/ },
+        FORBID_CONTENTS: [""] 
+      };
+      const expected = '<svg></svg>';
+      let clean = DOMPurify.sanitize(dirty, config);
+      assert.contains(clean, expected);
     });
   };
 });
